// ----------------------------------------------------------------------------
// Projekt       : Analytica AnaGate Lua-API
// File          : Lua-CANMessage.hpp
// Author        : Stefan Welisch
// Copyright     : (C) 2009 by Analytica GmbH
// ----------------------------------------------------------------------------
// $Id:$
//
// $Log:$
//
// ----------------------------------------------------------------------------

#ifndef LUA_CAN_MESSAGE_HPP
#define LUA_CAN_MESSAGE_HPP

// Includes -------------------------------------------------------------------
#include <sys/types.h>
#include <vector>

// Defines --------------------------------------------------------------------
#define MIN_MEMORY_CALLBACK_DATA_KB   2048


// packing mode 1 Byte
//#pragma pack(1)

#ifndef USE_DEFAULT_PACKING
   // packing mode 1 Byte
#pragma pack(push,1)
#endif

/** Class implements a Lua-CAN message.
    @author  Stefan Welisch
    @version 1.0
*/

class CLuaCanMessage
{
   public:
      enum { MAX_CAN_LENGTH  = 8 };

      /** Format flags */
      enum EFormatFlags
      {
         FMT_EXTENDED  = 0x01,    ///< CAN identifier is estended format (29 bit)
         FMT_REMOTE    = 0x02,    ///< telegram is a remote telegram (no data)
         FMT_TIME      = 0x04,    ///< telegram includes timestamp
         FMT_TCP_MSG   = 0x08     ///< telegram was sending from another TCP client to anagate
      };

      /** Construktor. Creates a telegram with id 0 and an empty data buffer. */
      CLuaCanMessage();

      /** Constructor.
          @param nHandle       Handle from connection.
          @param nIdentifier   Telegram id.
          @param pcData        Pointer to data buffer.
          @param nDataLength   Length og data buffer.
          @param nFormatFlags  Format flags.
          @param nSeconds      Seconds.
          @param nMicroseconds Microseconds.
      */
      CLuaCanMessage( const unsigned int    nHandle,
                      const unsigned int    nIdentifier,
                      const unsigned char * pcData,
                      const int             nDataLength,
                      const int             nFormatFlags,
                      const unsigned int    nSeconds,
                      const unsigned int    nMicroseconds);

      /** Destructor */
      virtual ~CLuaCanMessage();


      /** Sets the CAN telegram buffer. A CAN telegram has a maximum of 8 data bytes.
            @param  pcData      Pointer to message buffer.
            @param  nDataLength Message length.
      */
      void SetCANData(const unsigned char * pcData, int nDataLength);


      /** Gets the number of data bytes in the CAN telegram.
         @return Buffer length.
      */
      int GetLength( void ) const {return m_nDataLength;};


      /** Gets a pointer to the data buffer of teh CAN telegram.
         @return Identifier.
      */
      const unsigned char * GetBuffer( void ) const {return m_acDataBuffer;};


      /** Gets the identifier of the CAN telegram.
         @return Identifier.
      */
      unsigned int GetIdentifier( void ) const {return m_nIdentifier;};

      
      /** Gets the flags of the CAN telegram.
      @return Seconds.
       */
      unsigned char GetFlags( void ) const {return m_nFormatFlags;};


      /** Gets the Seconds of the CAN telegram.
      @return Seconds.
       */
      unsigned int GetSeconds( void ) const {return m_nSeconds;};


      /** Gets the Microseconds of the CAN telegram.
      @return Microseconds.
       */
      unsigned int GetMicroseconds( void ) const {return m_nMicroseconds;};

   protected:
      unsigned int   m_nHandle;                          ///< Handle from connection
      unsigned int   m_nIdentifier;                      ///< Message id
      unsigned int   m_nSeconds;                         ///< Seconds of message receipt (since 1970)
      unsigned int   m_nMicroseconds;                    ///< Microseconds of time

      unsigned char  m_nDataLength;                      ///< Message length
      unsigned char  m_acDataBuffer[MAX_CAN_LENGTH];     ///< Data buffer
      unsigned char  m_nFormatFlags;                     ///< Format flags

   private:
};

#ifndef USE_DEFAULT_PACKING
   // packing mode default
#pragma pack(pop)
#endif

#endif // #ifndef LUA_CAN_MESSAGE_HPP










